# -*- python -*-

import cgi
from _TutorialBase import *

class dicts(TutorialBase):

    title = 'Converting URL/Form Data to Python Dicts'
    css_links = ['tutorial.css']

    fields = ['name', 'address', 'city', 'state', 'zip', 'email']
    
    query_vars = [('userinfo', {})]

    def write_content(self):

        if self.req.method == 'POST':
            self.writeln('Show me the ', A('overview', href=self.req.uri),
                         ' again.')
            self.process_form()

        else:
            self.writeln(expand_codes(overview))

        self.writeln(P(self.theform()))
        

    def process_form(self):
        div = DIV(klass='formoutput')
        div.append(STRONG('This servlet has been posted via a form:'), BR(), BR())

        for field in self.fields:
            div.append('%s: %s<br>' % (field, self.userinfo.get(field, '?')))

        self.writeln(CENTER(div))

    def theform(self):

        table = TABLE(klass='formdisplay')
        for field in self.fields:
            table.append(TR(TD('%s: ' % field),
                            TD(INPUT(type='text',
                                     name='userinfo[%s]' % field,
                                     value=self.userinfo.get(field, '')))))
        table.append(TR(TD(INPUT(type='submit'),
                           colspan=2, align='center')))
        
        return FORM(CENTER(table), method='POST')


overview = DIV(

    P('''This servlet demonstrates how you can use standard python
      dicts in processing forms.  If the C{DEFAULT} for a variable in
      C{query_vars} is a dict, as in:'''),

    P(SPAN("query_vars = [('userinfo', {})]", klass='codesample')),

    P('''then for each request, the servlet will search the C{form}
      instance variable for names of the form C{userinfo[}E{key}C{]}.
      For every such occurrence a new key will be inserted into the
      C{userinfo} dictionary.'''),

    P('''The dict specified as C{DEFAULT} does not have to be empty.
      If non-empty then the keys must be strings and their values must
      be strings or lists.  If the value is a string then it will be
      retrieved with C{form.getfirst}, if a list, it will be retrieved
      with C{form.getlist}.  Note: C{DEFAULT}s that are strings or
      lists are E{replaced} when found in C{form}, however, with
      dicts, when keys are found in C{form} that are not found in the
      default, they are E{merged} with default dict.'''),

    P('''The benefits of collecting form data under the umbrella of
      one dict are considerable: fewer instance variables to manage,
      resulting in fewer lines of cleaner code.  If you create a class
      that subclasses dict you can use an instance of that class as
      C{DEFAULT} which will completely encapsulate your form in an
      object!'''),

    H3('Example'),

    P('''In the form below we name each text INPUT element with the
      special syntax:'''),

    PRE(cgi.escape('''
     <input type="text" name="userinfo[name]">
     <input type="text" name="userinfo[address]">
     <input type="text" name="userinfo[city]">
     <input type="text" name="userinfo[state]">
     <input type="text" name="userinfo[zip]">
     <input type="text" name="userinfo[email]">''')),

    P('''When the servlet processes C{query_vars} it will create one
      dict with six elements.'''))
