# -*- python -*-

import cgi
from _TutorialBase import *

class okmethods(TutorialBase):

      title = 'Calling Arbitrary Methods'
      css_links = ['tutorial.css']


      def foo(self, *args):
            self.methodcalled = ('foo called with %d args: %s'
                                 % (len(args),
                                    ', '.join(["'%s'" % a for a in args])))

      def bar(self):
            self.methodcalled = 'bar'

      def baz(self):
            self.methodcalled = 'baz'

      def uhoh(self):
            self.writeln(expand_codes(uhoh))
            return True
            
      ok_methods_to_call = [foo, bar, uhoh]

      def prep(self):
            TutorialBase.prep(self)

            self.methodcalled = None

      def write_content(self):

            if self.req.method == 'POST':
                  self.writeln('Show me the ',
                               A('overview', href=self.req.uri),
                               ' again.')
            else:
                  self.writeln(expand_codes(overview))

            self.writeln(CENTER(DIV('Method called: ', self.methodcalled,
                                    klass='formoutput')),
                         BR(),

                         self.theform())

      def theform(self):

            table = TABLE(border=1, width='50%', klass='formdisplay')

            table.append(TR(TD('Call method foo: '),
                            TD(INPUT(type='submit',
                                     name='_call_foo()',
                                     value='foo'))),
                         TR(TD('Call method foo with arguments: '),
                            TD(INPUT(type='submit',
                                     name='_call_foo(1, 2,3 )',
                                     value='foo with args'))),
                         TR(TD('Call method bar: '),
                            TD(INPUT(type='submit',
                                     name='_call_bar()',
                                     value='bar'))),
                         TR(TD('''Call method baz (this will not work
                               because baz is not listed in
                               ok_methods_to_call): '''),
                            TD(INPUT(type='submit',
                                     name='_call_baz()',
                                     value='baz'))),

                         TR(TD('''Call method uhoh (this method will
                               leave the tutorial because the method
                               returns True):'''),

                            TD(INPUT(type='submit',
                                     name='_call_uhoh()',
                                     value='uhoh'))))
                            
            return FORM(CENTER(table), method='POST')

      

overview = DIV(

      P('''In the previous servlet the C{repsond} method was
        introduced.  In most cases your servlets will subclass,
        directly or indirectly, C{HTMLPage}, which implements
        C{respond} in this way:'''),

      PRE('''
    def respond(self):
        
        return Servlet.respond(self) or self.write_html()''',
          klass='codesample'),

      P('''You will recall that C{respond} is supposed to return true
        or false, indicating whether or not a response was generated,
        so the above code says, "call the superclass C{respond} and if
        it does not generate a response, then call C{write_html}."
        The next servlet will concentrate on C{write_html}.  This
        servlet will discuss what happens in C{Servlet.respond}.'''),

      P('''C{Servlet.respond} does not, itself, generate output.  It
        searches the C{form} instance variable for names of the form:'''),

      P('C{_call_}E{NAME}C{(}E{ARGS}C{)}', style='margin-left:.5in'),

      P('''If no such name matches the pattern it returns C{False}
        (and so, C{write_html} will be called).  NOTE: if the request
        is by method GET, C{respond} will E{always} return
        C{False}.'''),

      P('''If such a name exists and the request is by method POST and
        E{NAME} is the name of a method listed in the instance
        variable C{ok_methods_to_call}, then the method will be called
        with the optional arguments, E{ARGS}, and the return value of
        the method will be returned by C{respond}.'''),

      P('''C{ok_methods_to_call} should be a list of unbound methods,
        E{not} just their string names.  E{ARGS}, if specified, should
        be comma separated and extra care should be taken in handling
        the arguments: the string is split on a comma and the
        resulting list of E{unstripped} strings become the arguments to
        the method.'''),

      P('''This is probably all too confusing.  It is actually a
        fairly easy mechanism to understand and is best shown with an
        example.'''),

      P('''The form below has five submit buttons.  They are rendered
        by the following HTML:'''),

      PRE(cgi.escape('''
        <input type="submit" name="_call_foo()" value="foo"> ...
        <input type="submit" name="_call_foo(1, 2,3 )" value="foo with args"> ...
        <input type="submit" name="_call_bar()" value="bar"> ...
        <input type="submit" name="_call_baz()" value="baz"> ...
        <input type="submit" name="_call_uhoh()" value="uhoh"> ...''')),

      P('''Note how the submit buttons are given a name with the
        special syntax.  If you view the source you will see
        C{ok_methods_to_call} is set like this:'''),

      P(SPAN('ok_methods_to_call = [foo, bar, uhoh]', klass='codesample')),

      P('''Note that C{ok_methods_to_call} is not set with the string
        names of the methods, but with the actual unbound method
        objects.  Now, submit the form with the various buttons and
        see how each of the methods is called.  C{foo} and C{bar} have
        no explicit return, so they implicitly return None, which
        evaluates to false, so C{write_html} will be called E{after}
        these methods.  C{baz} is not listed in C{ok_methods_to_call}
        so C{baz} will not be called at all.  C{uhoh} returns C{True},
        so this means C{write_html} will not be called.'''),

      HR())

uhoh = ''' Method uhoh returned True, so write_html was not called.
Click on your back button to return to the tutorial.'''

