# -*- python -*-

from _TutorialBase import *

class qvarsfvars(TutorialBase):

      title = 'Query Variables vs. Form Variables'
      css_links = ['tutorial.css']

      query_vars = [('qvar', 'qvar default')]
      form_vars  = [('fvar', 'fvar default')]

      def write_content(self):
            self.writeln(self.overview())

            self.writeln(expand_codes(CENTER(
                  P('Here are their values for this request:', BR(),
                    "Value of S{qvar}: '%s'" % self.qvar,
                    '&nbsp;' * 10,
                    "Value of S{fvar}: '%s'" % self.fvar,
                    klass='formoutput'))))
                         

      def overview(self):

            return expand_codes(DIV(
                  
                  
      P('In a ', A('previous servlet', href='formdata'), ''' both
         S{query variables} and S{form variables} were defined.  Since
         then, we have only used C{query_vars} to demonstrate
         processing URL and form data.  Now we will look at
         C{form_vars}.'''),

      P('''C{Form_vars} is identical to C{query_vars} except for one
        detail: while C{query_vars} is processed for all requests,
        C{form_vars} is only processed for S{POST} requests.  For
        S{GET} requests, any variable in C{form_vars} is set to its
        default value, regardless of whether or not it is in the URL
        or form data.  You may be wondering why this distinction is
        made and why this functionality exists.  Here are two
        examples:'''),

      OL(LI('''There may be applications where you do not want users
            to submit sensitive data with an URL (like usernames and
            passwords), but only through your form submit buttons.
            Using C{form_vars} can aid in enforcing this
            functionality.'''),

         LI('''Imagine a list of options a user can choose from
            (checkboxes, multiple selects), but you want to offer a
            default selection.  With only C{query_vars} there is no
            way to distinguish between a user deselecting all options
            from a user viewing a page for the first time.  Using
            C{form_vars} solves this problem: first time viewing is by
            GET, so defaults will always be used, but when the form is
            POSTed, the value will be retrieved with C{form.getlist},
            which may return an empty list, if all are deselected.''')),

      P('''Below are three ways of setting two variables, C{qvar} and
        C{fvar}, both to the value "1": setting variables with a URL
        (via GET) and setting the variables with forms (one via GET,
        one via POST).  You will notice that C{qvar} is set each time,
        but C{fvar} is only set with a POST:'''),


      CENTER(TABLE(TR(TD(A('Set qvar and fvar to "1"', href='?qvar=1&fvar=1')),

               TD('&nbsp;' *10),

               TD(FORM(INPUT(type='hidden', name='qvar', value='1'),
                       INPUT(type='hidden', name='fvar', value='1'),
                       INPUT(type='submit', value='GET submit'),
                       method='GET')),

               TD('&nbsp;' *10),
      
               TD(FORM(INPUT(type='hidden', name='qvar', value='1'),
                       INPUT(type='hidden', name='fvar', value='1'),
                       INPUT(type='submit', value='POST submit'),
                       action=self.req.uri, method='POST'))),
            klass='formdisplay'))))
